local gravity = 150 -- Gravitational constant

-- Player 'object'
local player = {
   x  = 450,
   y  = 450,
   vx = 0,
   vy = 0,
   moveSpeed   = 280,
   rotation    = 0,
   rotateSpeed = 6,
}

-- Enemy 'object'
local enemy = {
   x  = 450,
   y  = 100,
   vx = 0,
   vy = 0,
   moveSpeed   = 290,
   rotation    = 0,
   rotateSpeed = 6,

   targetX = 0,
   targetY = 0,
}


-- Draws a polygonal "triangle" as our ships
function drawShip(x, y, s, r)
   local x1, y1 = x + math.cos(r) * s, y + math.sin(r) * s
   local x2, y2 = x + math.cos(r + math.pi * 0.65) * s, y + math.sin(r + math.pi * 0.65) * s
   local x3, y3 = x, y
   local x4, y4 = x + math.cos(r - math.pi * 0.65) * s, y + math.sin(r - math.pi * 0.65) * s

   love.graphics.polygon("fill", x1, y1, x2, y2, x3, y3, x4, y4)
end

-- This is to make the game slow down in the beginning
local dtMult = 0

function love.update(dt)
   -- This is to make the game slow down in the beginning
   if dtMult < 1 then
      dtMult = dtMult + dt * 0.5
      dt = dt * dtMult
   end

   -- Take keyboard input and rotate player accordingly
   if love.keyboard.isDown("q") then
      player.rotation = player.rotation - player.rotateSpeed * dt
   end
   if love.keyboard.isDown("e") then
      player.rotation = player.rotation + player.rotateSpeed * dt
   end

   -- Calculate velocities of player and enemy
   player.vx = player.moveSpeed * math.cos(player.rotation)
   player.vy = player.moveSpeed * math.sin(player.rotation)

   enemy.vx = enemy.moveSpeed * math.cos(enemy.rotation)
   enemy.vy = enemy.moveSpeed * math.sin(enemy.rotation)

   -- Apply gravity to player and enemy
   player.vy = player.vy + gravity

   enemy.vy = enemy.vy + gravity

   -- Apply velocities to player and enemy
   player.x = player.x + player.vx * dt
   player.y = player.y + player.vy * dt

   enemy.x = enemy.x + enemy.vx * dt
   enemy.y = enemy.y + enemy.vy * dt

   -- Here's where we calculate the target position

   -- First we calculate an estimated time to the player
   local dx, dy = player.x - enemy.x, player.y - enemy.y
   local distance       = math.sqrt(dx * dx + dy * dy)
   local distanceInTime = distance / enemy.moveSpeed

   -- With this 'delta' time we can compenstate the target position
   -- Basically, we create a prediction as to where the player would be at the moment we'd reach him
   enemy.targetX = player.x + (player.moveSpeed * math.cos(player.rotation)) * distanceInTime
   enemy.targetY = player.y + (player.moveSpeed * math.sin(player.rotation) + gravity) * distanceInTime

   -- Calculate the rotation towards the target
   local targetRotation = math.atan2(enemy.targetY - enemy.y, enemy.targetX - enemy.x)

   -- Rotate enemy towards target
   if targetRotation > enemy.rotation then
      enemy.rotation = enemy.rotation + enemy.rotateSpeed * dt
   elseif targetRotation < enemy.rotation then
      enemy.rotation = enemy.rotation - enemy.rotateSpeed * dt
   end

   --[[ Some ways this could be made better:
         Smooth rotation
         Interpolating the target position, so it lags slightly behind
         Making flying more realistic in general
         When rotating towards the target, find the fastest way to rotate towards it. Right now it always tries to stay 0 - pi*2
         Some kind of fix so the enemy doesn't get confused by flying down while above him
           - This is caused by the enemy being between the player and the target
           - guess if you could find a way to check that and it return true, the enemy should aim straight for the player
   ]]
end

function love.draw()
   -- Draw the player in white
   love.graphics.setColor(255, 255, 255)
   drawShip(player.x, player.y, 10, player.rotation)

   -- Draw the enemy in red
   love.graphics.setColor(255, 0, 0)
   drawShip(enemy.x, enemy.y, 10, enemy.rotation)

   -- Draw the target
   love.graphics.circle("line", enemy.targetX, enemy.targetY, 3)

   -- Some debug text
   love.graphics.setColor(255, 255, 255)
   love.graphics.print("Turn with Q/E")
   love.graphics.print("Target position: " ..math.floor(enemy.targetX).. " " ..math.floor(enemy.targetY), 0, 15)

end
